@@ -18,13 +18,13 @@
 # along with Checkbox.  If not, see <http://www.gnu.org/licenses/>.
 
 """
-plainbox
-========
+:mod:`plainbox` -- main package
+===============================
 
 Simple checkbox redesign, without the complex message passing
 
-All public API is in the 'public' module.
-All abstract base classes are in the 'abc' module.
+All public API is in :mod:`plainbox.public`.
+All abstract base classes are in :mod:`plainbox.abc`.
 """
 
 import sys
@@ -35,4 +35,4 @@ if sys.version_info[0:2] < (3, 2):
 # PEP386 compliant version declaration.
 #
 # This is used by @public decorator to enforce our public API guarantees.
-__version__ = (0, 2, 0, "dev", 0)
+__version__ = (0, 3, 0, "final", 0)
