@@ -18,10 +18,8 @@
 # along with Checkbox.  If not, see <http://www.gnu.org/licenses/>.
 
 """
-plainbox.abc
-===============
-
-Abstract base classes used by checkbox
+:mod:`plainbox.abc` -- abstract base classes
+============================================
 
 Those classes are actually implemented in the plainbox.impl package. This
 module is here so that the essential API concepts are in a single spot and are
@@ -67,6 +65,8 @@ class IJobDefinition(metaclass=ABCMeta):
     def requires(self):
         """
         List of expressions that need to be true for this job to be available
+
+        This value can be None
         """
 
     @abstractproperty
@@ -76,6 +76,8 @@ class IJobDefinition(metaclass=ABCMeta):
 
         The return code, standard output and standard error streams are
         automatically recorded and processed, depending on the plugin type.
+
+        This value can be None
         """
 
     @abstractproperty
@@ -85,29 +87,31 @@ class IJobDefinition(metaclass=ABCMeta):
 
         This field is typically used to include execution and verification
         steps for manual and human-assisted tests.
+
+        This value can be None
         """
 
     @abstractproperty
     def depends(self):
         """
-        A list of jobs this job depends on
+        Comma-delimited dependency expression
+
+        This field can be used to express job dependencies. If a job depends on
+        another job it can only start if the other job had ran and succeeded.
+
+        This is the original data as provided when constructed. Use
+        get_direct_dependencies() to obtain the parsed equivalent.
+
+        This value can be None
         """
 
 
-class ITestResult(metaclass=ABCMeta):
+class IJobResult(metaclass=ABCMeta):
     """
-    Class for representing test results from a single test job
+    Class for representing results from a single job
     """
 
-    OUTCOME_UNSELECTED = 'unselected'
-    OUTCOME_UNSUPPORTED = 'unsupported'
-    OUTCOME_PASS = 'pass'
-    OUTCOME_FAIL = 'fail'
-    OUTCOME_SKIP = 'skip'
-    # XXX: we could have OUTCOME_UNKNOWN for manual tests if we wish to present
-    # that option to the user.
-
-    # XXX: We could also store stuff like test duration and other meta-data but
+    # XXX: We could also store stuff like job duration and other meta-data but
     # I wanted to avoid polluting this proposal with mundane details
 
     @abstractproperty
@@ -121,7 +125,7 @@ class ITestResult(metaclass=ABCMeta):
     @abstractproperty
     def outcome(self):
         """
-        Outcome of the test
+        Outcome of the test.
 
         The result of either automatic or manual verification. Depending on the
         plugin (test type). Available values are defined as class properties
@@ -178,9 +182,9 @@ class IJobRunner(metaclass=ABCMeta):
         Calling this method may block for arbitrary amount of time. User
         interfaces should ensure that it runs in a separate thread.
 
-        The return value is a TestResult object that contains all the data that
+        The return value is a JobResult object that contains all the data that
         was captured during the execution of the job. Some jobs may not return
-        a TestResult value.
+        a JobResult value.
         """
         # XXX: threads suck, could we make this fully asynchronous? The only
         # thing that we really want is to know when the command has stopped
@@ -188,23 +192,6 @@ class IJobRunner(metaclass=ABCMeta):
         # QT/GTK applications could tie that directly into their event loop.
 
 
-class IResourceContext(metaclass=ABCMeta):
-    """
-    Class used for storing and evaluating job requirement expressions.
-    """
-
-    @abstractproperty
-    def resources(self):
-        """
-        A namespace of resources.
-
-        The return value is a Namespace object that can be freely manipulated.
-        This object is used as a global environment for evaluating expressions.
-
-        The value of each resource should be a list or Namespace objects.
-        """
-
-
 class IUserInterfaceIO(metaclass=ABCMeta):
     """
     Base class that allows job runner to interact with the user interface.
