@@ -18,18 +18,23 @@
 # along with Checkbox.  If not, see <http://www.gnu.org/licenses/>.
 
 """
-plainbox.impl.rfc822
-====================
+:mod:`plainbox.impl.rfc822` -- RFC822 parser
+============================================
 
 Implementation of rfc822 serializer and deserializer.
 
- * THIS MODULE DOES NOT HAVE STABLE PUBLIC API *
+.. warning::
+
+    THIS MODULE DOES NOT HAVE STABLE PUBLIC API
 """
 
 import logging
 
 from inspect import cleandoc
 
+from plainbox.impl.secure.checkbox_trusted_launcher import RFC822SyntaxError
+from plainbox.impl.secure.checkbox_trusted_launcher import BaseRFC822Record
+
 logger = logging.getLogger("plainbox.rfc822")
 
 
@@ -37,10 +42,11 @@ class Origin:
     """
     Simple class for tracking where something came from
 
-    It has three attributes:
-        filename - the name of the file
-        line_start - the number of the line where the record begins
-        line_end - the number of the line where the record ends
+    :ivar filename: the name of the file
+
+    :ivar line_start: the number of the line where the record begins
+
+    :ivar line_end: the number of the line where the record ends
     """
 
     __slots__ = ['filename', 'line_start', 'line_end']
@@ -59,7 +65,7 @@ class Origin:
             self.filename, self.line_start, self.line_end)
 
 
-class RFC822Record:
+class RFC822Record(BaseRFC822Record):
     """
     Class for tracking RFC822 records
 
@@ -82,24 +88,6 @@ class RFC822Record:
         """
         return self._origin
 
-    @property
-    def data(self):
-        """
-        The data set (dictionary)
-        """
-        return self._data
-
-
-class RFC822SyntaxError(SyntaxError):
-    """
-    SyntaxError subclass for RFC822 parsing functions
-    """
-
-    def __init__(self, filename, lineno, msg):
-        self.filename = filename
-        self.lineno = lineno
-        self.msg = msg
-
 
 def load_rfc822_records(stream, data_cls=dict):
     """
@@ -118,7 +106,7 @@ def load_rfc822_records(stream, data_cls=dict):
 
 def gen_rfc822_records(stream, data_cls=dict):
     """
-    Load a sequence of rtf822-like records from a text stream.
+    Load a sequence of rfc822-like records from a text stream.
 
     Each record consists of any number of key-value pairs. Subsequent records
     are separated by one blank line. A record key may have a multi-line value
@@ -256,3 +244,33 @@ def gen_rfc822_records(stream, data_cls=dict):
     if data:
         logger.debug("yielding record: %r", record)
         yield record
+
+
+def dump_rfc822_records(message, stream):
+    """Dump a message to the output stream.
+
+    :param message: Dictionary containing message key/values.
+    :param stream: Output stream.
+    """
+    def _dump_part(stream, key, values):
+        stream.write("%s:\n" % key)
+        for value in values:
+            if not value:
+                stream.write(" .\n")
+            elif value == ".":
+                stream.write(" ..\n")
+            else:
+                stream.write(" %s\n" % value)
+
+    for key, value in message.items():
+        if isinstance(value, (list, tuple)):
+            _dump_part(stream, key, value)
+        elif isinstance(value, str) and "\n" in value:
+            values = value.split("\n")
+            if not values[-1]:
+                values = values[:-1]
+            _dump_part(stream, key, values)
+        else:
+            stream.write("%s: %s\n" % (key, value))
+
+    stream.write("\n")
