@@ -1,6 +1,6 @@
 # This file is part of Checkbox.
 #
-# Copyright 2012 Canonical Ltd.
+# Copyright 2012, 2013 Canonical Ltd.
 # Written by:
 #   Zygmunt Krynicki <zygmunt.krynicki@canonical.com>
 #
@@ -18,18 +18,20 @@
 # along with Checkbox.  If not, see <http://www.gnu.org/licenses/>.
 
 """
-plainbox.impl.session
-=====================
+:mod:`plainbox.impl.session` -- session state handling
+======================================================
 
-Internal implementation of plainbox
+.. warning::
 
- * THIS MODULE DOES NOT HAVE STABLE PUBLIC API *
+    THIS MODULE DOES NOT HAVE STABLE PUBLIC API
 """
+import json
 import logging
+import os
 import shutil
 import tempfile
 
-from plainbox.impl.depmgr import DependencyError
+from plainbox.impl.depmgr import DependencyError, DependencyDuplicateError
 from plainbox.impl.depmgr import DependencySolver
 from plainbox.impl.job import JobDefinition
 from plainbox.impl.resource import ExpressionCannotEvaluateError
@@ -163,7 +165,7 @@ class JobState:
     """
     Class representing the state of a job in a session.
 
-    Contains two basic properties of each job (either of which can be None):
+    Contains two basic properties of each job:
 
         * the readiness_inhibitor_list that prevent the job form starting
         * the result (outcome) of the run (IJobResult)
@@ -221,7 +223,7 @@ class JobState:
             return self._result
 
         def fset(self, value):
-            if value.job is not self.job:
+            if value.job.get_checksum() != self.job.get_checksum():
                 raise ValueError("result job does not match")
             self._result = value
 
@@ -246,79 +248,205 @@ class JobState:
         else:
             return "job can be started"
 
+    def _get_persistance_subset(self):
+        # Don't save resource job results, fresh data are required
+        # so we can't reuse the old ones
+        # The inhibitor list needs to be recomputed as well, don't save it.
+        state = {}
+        state['_job'] = self._job
+        if self._job.plugin == 'resource':
+            state['_result'] = JobResult({
+                'job': self._job,
+                'outcome': JobResult.OUTCOME_NONE
+            })
+        else:
+            state['_result'] = self._result
+        return state
+
+    @classmethod
+    def from_json_record(cls, record):
+        """
+        Create a JobState instance from JSON record
+        """
+        obj = cls(record['_job'])
+        obj._readiness_inhibitor_list = [UndesiredJobReadinessInhibitor]
+        obj._result = record['_result']
+        return obj
+
 
 class SessionState:
     """
     Class representing all state needed during a single program session.
 
+    This is the central glue/entry-point for applications. It connects user
+    intents to the rest of the system / plumbing and keeps all of the state in
+    one place.
+
     The set of utility methods and properties allow applications to easily
     handle the lower levels of dependencies, resources and ready states.
 
-    Once instantiated with a list of known jobs it is ready to react to
-    UI-driven changes. It is expected that the user will select / unselect
-    and run jobs. This class can react to both actions by recomputing the
-    dependency graph and updating the read states accordingly.
+    SessionState has the following instance variables, all of which are
+    currently exposed as properties.
 
-    Ready states (one for each job) allow the UI to take simple decisions
-    (either can or cannot run)
+    :ivar list job_list: A list of all known jobs
+
+        Not all the jobs from this list are going to be executed (or selected
+        for execution) by the user.
+
+        It may change at runtime because of local jobs. Note that in upcoming
+        changes this will start out empty and will be changeable dynamically.
+        It can still change due to local jobs but there is no API yes.
+
+        This list cannot have any duplicates, if that is the case a
+        DependencyDuplicateError is raised. This has to be handled externally
+        and is a sign that the job database is corrupted or has wrong data. As
+        an exception if duplicates are perfectly identical this error is
+        silently corrected.
+
+    :ivar dict job_state_map: mapping that tracks the state of each job
+
+        Mapping from job name to :class:`JobState`. This basically has the test
+        result and the inhibitor of each job. It also serves as a
+        :attr:`plainbox.impl.job.JobDefinition.name`-> job lookup helper.
+
+        Directly exposed with the intent to fuel part of the UI. This is a way
+        to get at the readiness state, result and readiness inhibitors, if any.
+
+        XXX: this can loose data job_list has jobs with the same name. It would
+        be better to use job id as the keys here. A separate map could be used
+        for the name->job lookup. This will be fixed when session controller
+        branch lands in trunk as then jobs are dynamically added to the system
+        one at a time and proper error conditions can be detected and reported.
+
+    :ivar list desired_job_list: subset of jobs selected for execution
+
+        This is used to compute :attr:`run_list`. It can only be changed by
+        calling :meth:`update_desired_job_list()` which returns meaningful
+        values so this is not a settable property.
+
+    :ivar list run_list: sorted list of jobs to execute
+
+        This is basically a superset of desired_job_list and a subset of
+        job_list that is topologically sorted to allowing all desired jobs to
+        run. This property is updated whenever desired_job_list is changed.
+
+    :ivar dict resource_map: all known resources
+
+        A mapping from resource name to a list of
+        :class:`plainbox.impl.resource.Resource` objects. This encapsulates all
+        "knowledge" about the system plainbox is running on.
+
+        It is needed to compute job readiness (as it stores resource data
+        needed by resource programs). It is also available to exporters.
+
+        This is computed internally from the output of checkbox resource jobs,
+        it can only be changed by calling :meth:`update_job_result()`
     """
 
+    session_data_filename = 'session.json'
+
     def __init__(self, job_list):
-        # The original list of job that the system knows about.
-        # Not all jobs from this list are going to be executed
-        # (or selected for execution) by the user.
+        # Start by making a copy of job_list as we may modify it below
+        job_list = job_list[:]
+        while True:
+            try:
+                # Construct a solver with the job list as passed by the caller.
+                # This will do a little bit of validation and might raise
+                # DepdendencyDuplicateError if there are any duplicates at this
+                # stage.
+                #
+                # There's a single case that is handled here though, if both
+                # jobs are identical this problem is silently fixed. This
+                # should not happen in normal circumstances but is non the less
+                # harmless (as long as both jobs are perfectly identical)
+                #
+                # Since this problem can happen any number of times (many
+                # duplicates) this is performed in a loop. The loop breaks when
+                # we cannot solve the problem _OR_ when no error occurs.
+                DependencySolver(job_list)
+            except DependencyDuplicateError as exc:
+                # If both jobs are identical then silently fix the problem by
+                # removing one of the jobs (here the second one we've seen but
+                # it's not relevant as they are possibly identical) and try
+                # again
+                if exc.job == exc.duplicate_job:
+                    job_list.remove(exc.duplicate_job)
+                    continue
+                else:
+                    # If the jobs differ report this back to the caller
+                    raise
+            else:
+                # If there are no problems then break the loop
+                break
         self._job_list = job_list
-        # State of each job, see JobState for details but it basically
-        # has the test result and the inhibitor of each job. It also serves
-        # as a job.name -> job lookup helper.
-        #
-        # Directly exposed with the intent to fuel part of the UI.
-        #
-        # XXX: this can loose data job_list has jobs with the same name. It
-        # would be better to use job id as the keys here. A separate map could
-        # be used for the name->job lookup.
-        self._job_state_map = {job.name: JobState(job) for job in job_list}
-        # A subset of job_list that was selected by the user for execution.
-        # Used to compute run_list. Can be changed at will during lifetime
-        # of this object
+        self._job_state_map = {job.name: JobState(job)
+                               for job in self._job_list}
         self._desired_job_list = []
-        # Copy of desired_job_list that was topologically sorted by the
-        # dependency solver. Jobs must run in this order (although not all jobs
-        # may actually run or will actually be successful)
         self._run_list = []
-        # A collection of known resources. Mapping resource job name to a list
-        # of resource objects. Needed to compute task readiness (as it stores
-        # resource data needed by resource programs). Currently not exposed
-        # outside of this class.
         self._resource_map = {}
         # Temporary directory used as 'scratch space' for running jobs. Removed
         # entirely when session is terminated. Internally this is exposed as
         # $CHECKBOX_DATA to script environment.
         self._session_dir = None
 
+        # Directory used to store jobs IO logs.
+        self._jobs_io_log_dir = None
+
+    def _get_persistance_subset(self):
+        state = {}
+        state['_job_state_map'] = self._job_state_map
+        state['_desired_job_list'] = self._desired_job_list
+        return state
+
+    @classmethod
+    def from_json_record(cls, record):
+        """
+        Create a SessionState instance from JSON record
+        """
+        obj = cls([])
+        obj._job_state_map = record['_job_state_map']
+        obj._desired_job_list = record['_desired_job_list']
+        return obj
+
     def open(self):
         """
         Open session state for running jobs.
 
-        This function creates a temporary directory where jobs can store their
-        temporary data. This directory will be removed by .close()
+        This function creates the cache directory where jobs can store their
+        data. See:
+        http://standards.freedesktop.org/basedir-spec/basedir-spec-latest.html
         """
         if self._session_dir is None:
-            self._session_dir = tempfile.mkdtemp()
+            xdg_cache_home = os.environ.get('XDG_CACHE_HOME') or \
+                os.path.join(os.path.expanduser('~'), '.cache')
+            self._session_dir = os.path.join(xdg_cache_home, 'plainbox')
+            if not os.path.isdir(self._session_dir):
+                os.makedirs(self._session_dir)
+        if self._jobs_io_log_dir is None:
+            self._jobs_io_log_dir = os.path.join(self._session_dir, 'io-logs')
+            if not os.path.isdir(self._jobs_io_log_dir):
+                os.makedirs(self._jobs_io_log_dir)
         return self
 
-    def close(self):
+    def clean(self):
         """
-        Close the session and remove temporary disk state.
-
-        This function removes the directory created by .open() and all the data
-        that was placed there. It is automatically called by __exit__, the
-        context manager exit function. Care should be taken to ensure that all
-        session data, particularly attachments, were saved before.
+        Clean the session directory.
         """
         if self._session_dir is not None:
             shutil.rmtree(self._session_dir)
             self._session_dir = None
+            self._jobs_io_log_dir = None
+            self.open()
+
+    def close(self):
+        """
+        Close the session.
+
+        It is automatically called by __exit__, the context manager exit
+        function.
+        """
+        self._session_dir = None
+        self._jobs_io_log_dir = None
 
     def update_desired_job_list(self, desired_job_list):
         """
@@ -347,7 +475,7 @@ class SessionState:
             # resources or runtime complexity.
             try:
                 self._run_list = DependencySolver.resolve_dependencies(
-                    self._job_list, desired_job_list)
+                    self._job_list, self._desired_job_list)
             except DependencyError as exc:
                 # When a dependency error is detected remove the affected job
                 # form _desired_job_list and try again.
@@ -399,6 +527,121 @@ class SessionState:
         # Update all job readiness state
         self._recompute_job_readiness()
 
+    def previous_session_file(self):
+        """
+        Check the filesystem for previous session data
+        Returns the full pathname to the session file if it exists
+        """
+        session_filename = os.path.join(self._session_dir,
+                                        self.session_data_filename)
+        if os.path.exists(session_filename):
+            return session_filename
+        else:
+            return None
+
+    def add_job(self, new_job):
+        """
+        Add a new job to the session
+
+        :param new_job: the job being added
+
+        :raises DependencyDuplicateError:
+            if a duplicate, clashing job definition is detected
+
+        The new_job gets added to all the state tracking objects of the
+        session.  The job is initially not selected to run (it is not in the
+        desired_job_list and has the undesired inhibitor).
+
+        The new_job may clash with an existing job with the same name. Unless
+        both jobs are identical this will cause DependencyDuplicateError to be
+        raised. Identical jobs are silently discarded.
+
+        .. note::
+
+            This method recomputes job readiness for all jobs
+        """
+        # See if we have a job with the same name already
+        try:
+            existing_job = self._job_state_map[new_job.name].job
+        except KeyError:
+            logger.info("Storing new job %r", new_job)
+            # Register the new job in our state
+            self._job_state_map[new_job.name] = JobState(new_job)
+            self._job_list.append(new_job)
+        else:
+            # If there is a clash report DependencyDuplicateError only when the
+            # hashes are different. This prevents a common "problem" where
+            # "__foo__" local jobs just load all jobs from the "foo" category.
+            if new_job != existing_job:
+                raise DependencyDuplicateError(existing_job, new_job)
+        # Update all job readiness state
+        self._recompute_job_readiness()
+
+    def set_resource_list(self, resource_name, resource_list):
+        """
+        Add or change a resource with the given name.
+
+        Resources silently overwrite any old resources with the same name.
+        """
+        self._resource_map[resource_name] = resource_list
+
+    def persistent_save(self):
+        """
+        Save to disk the minimum needed to resume plainbox where it stopped
+        """
+        # Ensure an atomic update of the session file:
+        #   - create a new temp file (on the same file system!)
+        #   - write data to the temp file
+        #   - fsync() the temp file
+        #   - rename the temp file to the appropriate name
+        #   - fsync() the containing directory
+        # Calling fsync() does not necessarily ensure that the entry in the
+        # directory containing the file has also reached disk.
+        # For that an explicit fsync() on a file descriptor for the directory
+        # is also needed.
+        filename = os.path.join(self._session_dir,
+                                self.session_data_filename)
+
+        with tempfile.NamedTemporaryFile(mode='wt',
+                                         encoding='UTF-8',
+                                         suffix='.tmp',
+                                         prefix='session',
+                                         dir=self._session_dir,
+                                         delete=False) as tmpstream:
+            # Save the session state to disk
+            json.dump(self, tmpstream, cls=SessionStateEncoder,
+                      ensure_ascii=False, indent=None, separators=(',', ':'))
+
+            tmpstream.flush()
+            os.fsync(tmpstream.fileno())
+
+        session_dir_fd = os.open(self._session_dir, os.O_DIRECTORY)
+        os.rename(tmpstream.name, filename)
+        os.fsync(session_dir_fd)
+        os.close(session_dir_fd)
+
+    def resume(self):
+        """
+        Erase the job_state_map and desired_job_list with the saved ones
+        """
+        with open(self.previous_session_file(), 'rt', encoding='UTF-8') as f:
+            previous_session = json.load(
+                f, object_hook=SessionStateEncoder().dict_to_object)
+        self._job_state_map = previous_session._job_state_map
+        desired_job_list = []
+        for job in previous_session._desired_job_list:
+            if job in self._job_list:
+                desired_job_list.extend(
+                    [j for j in self._job_list if j == job])
+            elif (previous_session._job_state_map[job.name].result.outcome !=
+                    JobResult.OUTCOME_NONE):
+                # Keep jobs results from the previous session without a
+                # definition in the current job_list only if they have
+                # a valid result
+                desired_job_list.append(job)
+        self.update_desired_job_list(desired_job_list)
+        # FIXME: Restore io_logs from files
+
     def _process_resource_result(self, result):
         new_resource_list = []
         for record in self._gen_rfc822_records_from_io_log(result):
@@ -416,13 +659,13 @@ class SessionState:
         # name, not a new list of jobs)
         new_job_list = []
         for record in self._gen_rfc822_records_from_io_log(result):
-            new_job = JobDefinition.from_rfc822_record(record)
+            new_job = result.job.create_child_job_from_record(record)
             new_job_list.append(new_job)
         # Then for each new job, add it to the job_list, unless it collides
         # with another job with the same name.
         for new_job in new_job_list:
             try:
-                existing_job = self._job_state_map[new_job.name]
+                existing_job = self._job_state_map[new_job.name].job
             except KeyError:
                 logger.info("Storing new job %r", new_job)
                 self._job_state_map[new_job.name] = JobState(new_job)
@@ -432,15 +675,17 @@ class SessionState:
                 # reported back to the UI layer. Perhaps update_job_result()
                 # could simply return a list of problems in a similar manner
                 # how update_desired_job_list() does.
-                logging.warning(
-                    ("Local job %s produced job %r that collides with"
-                     " an existing job %r, the new job was discarded"),
-                    result.job, new_job, existing_job)
+                if new_job != existing_job:
+                    logging.warning(
+                        ("Local job %s produced job %r that collides with"
+                         " an existing job %r, the new job was discarded"),
+                        result.job, new_job, existing_job)
 
     def _gen_rfc822_records_from_io_log(self, result):
         logger.debug("processing output from a job: %r", result.job)
         # Select all stdout lines from the io log
-        line_gen = (record[2] for record in result.io_log
+        line_gen = (record[2].decode('UTF-8', errors='replace')
+                    for record in result.io_log
                     if record[1] == 'stdout')
         try:
             # Parse rfc822 records from the subsequent lines
@@ -502,6 +747,15 @@ class SessionState:
         """
         return self._session_dir
 
+    @property
+    def jobs_io_log_dir(self):
+        """
+        pathname of the jobs IO logs directory
+
+        This is not None only between calls to open() / close().
+        """
+        return self._jobs_io_log_dir
+
     def _recompute_job_readiness(self):
         """
         Internal method of SessionState.
@@ -561,7 +815,7 @@ class SessionState:
                         related_expression=exc.expression)
                     job_state.readiness_inhibitor_list.append(inhibitor)
             # Check if all job dependencies ran successfully
-            for dep_name in job.get_direct_dependencies():
+            for dep_name in sorted(job.get_direct_dependencies()):
                 dep_job_state = self._job_state_map[dep_name]
                 # If the dependency did not have a chance to run yet add the
                 # PENDING_DEP inhibitor.
@@ -582,8 +836,45 @@ class SessionState:
                     job_state.readiness_inhibitor_list.append(inhibitor)
 
     def __enter__(self):
-        self.open()
         return self
 
     def __exit__(self, *args):
         self.close()
+
+
+class SessionStateEncoder(json.JSONEncoder):
+
+    _class_indentifiers = {
+        JobDefinition: 'JOB_DEFINITION',
+        JobResult: 'JOB_RESULT',
+        JobState: 'JOB_STATE',
+        SessionState: 'SESSION_STATE',
+    }
+
+    def default(self, obj):
+        """
+        JSON Serialize helper to encode SessionState attributes
+        Convert objects to a dictionary of their representation
+        """
+        if (isinstance(obj, (JobDefinition, JobResult, JobState,
+                             SessionState))):
+            d = {'_class_id': self._class_indentifiers[obj.__class__]}
+            d.update(obj._get_persistance_subset())
+            return d
+        else:
+            return json.JSONEncoder.default(self, obj)
+
+    def dict_to_object(self, d):
+        """
+        JSON Decoder helper
+        Convert dictionary to python objects
+        """
+        if '_class_id' in d:
+            for c, id in self._class_indentifiers.items():
+                if id == d['_class_id']:
+                    cls = c
+                    inst = cls.from_json_record(d)
+                    break
+        else:
+            inst = d
+        return inst
