@@ -29,8 +29,9 @@ from unittest import TestCase
 
 from plainbox.impl.rfc822 import Origin
 from plainbox.impl.rfc822 import RFC822Record
-from plainbox.impl.rfc822 import RFC822SyntaxError
 from plainbox.impl.rfc822 import load_rfc822_records
+from plainbox.impl.rfc822 import dump_rfc822_records
+from plainbox.impl.secure.checkbox_trusted_launcher import RFC822SyntaxError
 
 
 class OriginTests(TestCase):
@@ -64,16 +65,18 @@ class RFC822RecordTests(TestCase):
         self.assertEqual(record.origin, origin)
 
 
-class RFC822ParserTests(TestCase):
+class RFC822ParserTestsMixIn():
+
+    loader = load_rfc822_records
 
     def test_empty(self):
         with StringIO("") as stream:
-            records = load_rfc822_records(stream)
+            records = type(self).loader(stream)
         self.assertEqual(len(records), 0)
 
     def test_single_record(self):
         with StringIO("key:value") as stream:
-            records = load_rfc822_records(stream)
+            records = type(self).loader(stream)
         self.assertEqual(len(records), 1)
         self.assertEqual(records[0].data, {'key': 'value'})
 
@@ -93,7 +96,7 @@ class RFC822ParserTests(TestCase):
             "\n"
         )
         with StringIO(text) as stream:
-            records = load_rfc822_records(stream)
+            records = type(self).loader(stream)
         self.assertEqual(len(records), 3)
         self.assertEqual(records[0].data, {'key1': 'value1'})
         self.assertEqual(records[1].data, {'key2': 'value2'})
@@ -108,7 +111,7 @@ class RFC822ParserTests(TestCase):
             "key3:value3\n"
         )
         with StringIO(text) as stream:
-            records = load_rfc822_records(stream)
+            records = type(self).loader(stream)
         self.assertEqual(len(records), 3)
         self.assertEqual(records[0].data, {'key1': 'value1'})
         self.assertEqual(records[1].data, {'key2': 'value2'})
@@ -121,7 +124,7 @@ class RFC822ParserTests(TestCase):
             " value\n"
         )
         with StringIO(text) as stream:
-            records = load_rfc822_records(stream)
+            records = type(self).loader(stream)
         self.assertEqual(len(records), 1)
         self.assertEqual(records[0].data, {'key': 'longer\nvalue'})
 
@@ -133,7 +136,7 @@ class RFC822ParserTests(TestCase):
             " value\n"
         )
         with StringIO(text) as stream:
-            records = load_rfc822_records(stream)
+            records = type(self).loader(stream)
         self.assertEqual(len(records), 1)
         self.assertEqual(records[0].data, {'key': 'longer\n\nvalue'})
 
@@ -145,7 +148,7 @@ class RFC822ParserTests(TestCase):
             " value\n"
         )
         with StringIO(text) as stream:
-            records = load_rfc822_records(stream)
+            records = type(self).loader(stream)
         self.assertEqual(len(records), 1)
         self.assertEqual(records[0].data, {'key': 'longer\n.\nvalue'})
 
@@ -160,7 +163,7 @@ class RFC822ParserTests(TestCase):
             " value 2\n"
         )
         with StringIO(text) as stream:
-            records = load_rfc822_records(stream)
+            records = type(self).loader(stream)
         self.assertEqual(len(records), 2)
         self.assertEqual(records[0].data, {'key1': 'initial\nlonger\nvalue 1'})
         self.assertEqual(records[1].data, {'key2': 'longer\nvalue 2'})
@@ -168,7 +171,7 @@ class RFC822ParserTests(TestCase):
     def test_irrelevant_whitespace(self):
         text = "key :  value  "
         with StringIO(text) as stream:
-            records = load_rfc822_records(stream)
+            records = type(self).loader(stream)
         self.assertEqual(len(records), 1)
         self.assertEqual(records[0].data, {'key': 'value'})
 
@@ -178,7 +181,7 @@ class RFC822ParserTests(TestCase):
             " value\n"
         )
         with StringIO(text) as stream:
-            records = load_rfc822_records(stream)
+            records = type(self).loader(stream)
         self.assertEqual(len(records), 1)
         self.assertEqual(records[0].data, {'key': 'value'})
 
@@ -186,21 +189,21 @@ class RFC822ParserTests(TestCase):
         text = " extra value"
         with StringIO(text) as stream:
             with self.assertRaises(RFC822SyntaxError) as call:
-                load_rfc822_records(stream)
+                type(self).loader(stream)
             self.assertEqual(call.exception.msg, "Unexpected multi-line value")
 
     def test_garbage(self):
         text = "garbage"
         with StringIO(text) as stream:
             with self.assertRaises(RFC822SyntaxError) as call:
-                load_rfc822_records(stream)
+                type(self).loader(stream)
             self.assertEqual(call.exception.msg, "Unexpected non-empty line")
 
     def test_syntax_error(self):
         text = "key1 = value1"
         with StringIO(text) as stream:
             with self.assertRaises(RFC822SyntaxError) as call:
-                load_rfc822_records(stream)
+                type(self).loader(stream)
             self.assertEqual(call.exception.msg, "Unexpected non-empty line")
 
     def test_duplicate_error(self):
@@ -210,7 +213,65 @@ class RFC822ParserTests(TestCase):
         )
         with StringIO(text) as stream:
             with self.assertRaises(RFC822SyntaxError) as call:
-                load_rfc822_records(stream)
+                type(self).loader(stream)
             self.assertEqual(call.exception.msg, (
                 "Job has a duplicate key 'key1' with old value 'value1'"
                 " and new value 'value2'"))
+
+
+class RFC822ParserTests(TestCase, RFC822ParserTestsMixIn):
+
+    pass
+
+
+class RFC822WriterTests(TestCase):
+
+    def test_single_record(self):
+        with StringIO() as stream:
+            dump_rfc822_records({'key': 'value'}, stream)
+            self.assertEqual(stream.getvalue(), "key: value\n\n")
+
+    def test_multiple_record(self):
+        with StringIO() as stream:
+            dump_rfc822_records({'key1': 'value1', 'key2': 'value2'}, stream)
+            self.assertIn(
+                stream.getvalue(), (
+                    "key1: value1\nkey2: value2\n\n",
+                    "key2: value2\nkey1: value1\n\n"))
+
+    def test_multiline_value(self):
+        text = (
+            "key:\n"
+            " longer\n"
+            " value\n\n"
+        )
+        with StringIO() as stream:
+            dump_rfc822_records({'key': 'longer\nvalue'}, stream)
+            self.assertEqual(stream.getvalue(), text)
+
+    def test_multiline_value_with_space(self):
+        text = (
+            "key:\n"
+            " longer\n"
+            " .\n"
+            " value\n\n"
+        )
+        with StringIO() as stream:
+            dump_rfc822_records({'key': 'longer\n\nvalue'}, stream)
+            self.assertEqual(stream.getvalue(), text)
+
+    def test_multiline_value_with_period(self):
+        text = (
+            "key:\n"
+            " longer\n"
+            " ..\n"
+            " value\n\n"
+        )
+        with StringIO() as stream:
+            dump_rfc822_records({'key': 'longer\n.\nvalue'}, stream)
+            self.assertEqual(stream.getvalue(), text)
+
+    def test_type_error(self):
+        with StringIO() as stream:
+            with self.assertRaises(AttributeError):
+                dump_rfc822_records(['key', 'value'], stream)
