@@ -1,6 +1,6 @@
 # This file is part of Checkbox.
 #
-# Copyright 2012 Canonical Ltd.
+# Copyright 2012, 2013 Canonical Ltd.
 # Written by:
 #   Zygmunt Krynicki <zygmunt.krynicki@canonical.com>
 #
@@ -24,16 +24,24 @@ plainbox.impl.test_session
 Test definitions for plainbox.impl.session module
 """
 
+import json
+import os
+import tempfile
+import shutil
+
+from tempfile import TemporaryDirectory
 from unittest import TestCase
 
+from plainbox.impl.depmgr import DependencyDuplicateError
+from plainbox.impl.depmgr import DependencyMissingError
 from plainbox.impl.resource import Resource
 from plainbox.impl.result import JobResult
 from plainbox.impl.session import JobReadinessInhibitor
 from plainbox.impl.session import JobState
 from plainbox.impl.session import SessionState
+from plainbox.impl.session import SessionStateEncoder
 from plainbox.impl.session import UndesiredJobReadinessInhibitor
-from plainbox.impl.testing_utils import make_job
-from plainbox.impl.testing_utils import make_job_result
+from plainbox.impl.testing_utils import make_io_log, make_job, make_job_result
 
 
 class JobReadinessInhibitorTests(TestCase):
@@ -175,6 +183,72 @@ class JobStateTests(TestCase):
             self.job_state.get_readiness_description().startswith(
                 "job cannot be started: "))
 
+    def test_encode_resource_job(self):
+        self.job_R = make_job("R", plugin="resource")
+        result_R = JobResult({
+            'job': self.job_R,
+            'outcome': JobResult.OUTCOME_PASS,
+            'io_log': ((0, 'stdout', "attr: value\n"),)
+        })
+        jobstate = JobState(self.job_R)
+        jobstate.result = result_R
+        jobstate_enc = jobstate._get_persistance_subset()
+        # The inhibitor list is not saved
+        with self.assertRaises(KeyError):
+            jobstate_enc['_readiness_inhibitor_list']
+        # Resource have to be re evealutated on startup, outcome of the job
+        # must be reset to JobResult.OUTCOME_NONE
+        self.assertEqual(jobstate_enc['_result'].outcome,
+                         JobResult.OUTCOME_NONE)
+
+    def test_encode_normal_job(self):
+        result = JobResult({
+            'job': self.job,
+            'outcome': JobResult.OUTCOME_PASS,
+        })
+        self.job_state.result = result
+        jobstate_enc = self.job_state._get_persistance_subset()
+        # The inhibitor list is not saved
+        with self.assertRaises(KeyError):
+            jobstate_enc['_readiness_inhibitor_list']
+        # Normal jobs should keep their outcome value
+        self.assertEqual(jobstate_enc['_result'].outcome,
+                         JobResult.OUTCOME_PASS)
+
+    def test_decode(self):
+        raw_json = """{
+            "_class_id": "JOB_STATE",
+            "_job": {
+                "_class_id": "JOB_DEFINITION",
+                "data": {
+                    "name": "X",
+                    "plugin": "dummy"
+                }
+            },
+            "_result": {
+                "_class_id": "JOB_RESULT",
+                "data": {
+                    "comments": null,
+                    "job": {
+                        "_class_id": "JOB_DEFINITION",
+                        "data": {
+                            "name": "X",
+                            "plugin": "dummy"
+                        }
+                    },
+                    "outcome": "pass",
+                    "return_code": null
+                }
+            }
+        }"""
+        job_dec = json.loads(
+            raw_json, object_hook=SessionStateEncoder().dict_to_object)
+        self.assertIsInstance(job_dec, JobState)
+        self.assertEqual(
+            repr(job_dec._result), (
+                "<JobResult job:<JobDefinition name:'X'"
+                " plugin:'dummy'> outcome:'pass'>"))
+
 
 class SessionStateSmokeTests(TestCase):
 
@@ -204,6 +278,133 @@ class SessionStateSmokeTests(TestCase):
         self.assertIsNone(self.session_state.session_dir)
 
 
+class RegressionTests(TestCase):
+    # Tests for bugfixes
+
+    def test_crash_in_update_desired_job_list(self):
+        # This checks if a DependencyError can cause crash
+        # update_desired_job_list() with a ValueError, in certain conditions.
+        A = make_job('A', depends='X')
+        L = make_job('L', plugin='local')
+        session = SessionState([A, L])
+        problems = session.update_desired_job_list([A, L])
+        # We should get exactly one DependencyMissingError related to job A and
+        # the undefined job X (that is presumably defined by the local job L)
+        self.assertEqual(len(problems), 1)
+        self.assertIsInstance(problems[0], DependencyMissingError)
+        self.assertIs(problems[0].affected_job, A)
+
+    def test_init_with_identical_jobs(self):
+        A = make_job("A")
+        second_A = make_job("A")
+        third_A = make_job("A")
+        # Identical jobs are folded for backwards compatibility with some local
+        # jobs that re-added existing jobs
+        session = SessionState([A, second_A, third_A])
+        # But we don't really store both, just the first one
+        self.assertEqual(session.job_list, [A])
+
+    def test_init_with_colliding_jobs(self):
+        # This is similar to the test above but the jobs actually differ In
+        # this case the _second_ job is rejected but it really signifies a
+        # deeper problem that should only occur during development of jobs
+        A = make_job("A")
+        different_A = make_job("A", plugin="resource")
+        with self.assertRaises(DependencyDuplicateError) as call:
+            SessionState([A, different_A])
+            self.assertIs(call.exception.job, A)
+            self.assertIs(call.exception.duplicate_job, different_A)
+            self.assertIs(call.exception.affected_job, different_A)
+
+
+class SessionStateAPITests(TestCase):
+
+    def test_set_resource_list(self):
+        # Define an empty session
+        session = SessionState([])
+        # Define a resource
+        old_res = Resource({'attr': 'old value'})
+        # Set the resource list with the old resource
+        # So here the old result is stored into a new 'R' resource
+        session.set_resource_list('R', [old_res])
+        # Ensure that it worked
+        self.assertEqual(session._resource_map, {'R': [old_res]})
+        # Define another resource
+        new_res = Resource({'attr': 'new value'})
+        # Now we present the second result for the same job
+        session.set_resource_list('R', [new_res])
+        # What should happen here is that the R resource is entirely replaced
+        # by the data from the new result. The data should not be merged or
+        # appended in any way.
+        self.assertEqual(session._resource_map, {'R': [new_res]})
+
+    def test_add_job(self):
+        # Define a job
+        job = make_job("A")
+        # Define an empty session
+        session = SessionState([])
+        # Add the job to the session
+        session.add_job(job)
+        # The job got added to job list
+        self.assertIn(job, session.job_list)
+        # The job got added to job state map
+        self.assertIs(session.job_state_map[job.name].job, job)
+        # The job is not added to the desired job list
+        self.assertNotIn(job, session.desired_job_list)
+        # The job is not in the run list
+        self.assertNotIn(job, session.run_list)
+        # The job is not selected to run
+        self.assertEqual(
+            session.job_state_map[job.name].readiness_inhibitor_list,
+            [UndesiredJobReadinessInhibitor])
+
+    def test_add_job_duplicate_job(self):
+        # Define a job
+        job = make_job("A")
+        # Define an empty session
+        session = SessionState([])
+        # Add the job to the session
+        session.add_job(job)
+        # The job got added to job list
+        self.assertIn(job, session.job_list)
+        # Define a perfectly identical job
+        duplicate_job = make_job("A")
+        self.assertEqual(job, duplicate_job)
+        # Try adding it to the session
+        #
+        # Note that this does not raise any exceptions as the jobs are perfect
+        # duplicates.
+        session.add_job(duplicate_job)
+        # The new job _did not_ get added to the job list
+        self.assertEqual(len(session.job_list), 1)
+        self.assertIsNot(duplicate_job, session.job_list[0])
+
+    def test_add_job_clashing_job(self):
+        # Define a job
+        job = make_job("A")
+        # Define an empty session
+        session = SessionState([])
+        # Add the job to the session
+        session.add_job(job)
+        # The job got added to job list
+        self.assertIn(job, session.job_list)
+        # Define a different job that clashes with the initial job
+        clashing_job = make_job("A", plugin='other')
+        self.assertNotEqual(job, clashing_job)
+        self.assertEqual(job.name, clashing_job.name)
+        # Try adding it to the session
+        #
+        # This raises an exception
+        with self.assertRaises(DependencyDuplicateError) as call:
+            session.add_job(clashing_job)
+        # The exception gets job in the right order
+        self.assertIs(call.exception.affected_job, job)
+        self.assertIs(call.exception.affecting_job, clashing_job)
+        # The new job _did not_ get added to the job list
+        self.assertEqual(len(session.job_list), 1)
+        self.assertIsNot(clashing_job, session.job_list[0])
+
+
 class SessionStateSpecialTests(TestCase):
 
     # NOTE: those tests are essential. They allow testing the behavior of
@@ -234,7 +435,7 @@ class SessionStateSpecialTests(TestCase):
     # most of their time adding (changing) jobs in an ever-growing pile that
     # they don't necessarily fully know, comprehend or remember.
 
-    def test_resource_job_affects_resouces(self):
+    def test_resource_job_affects_resources(self):
         pass
 
 
@@ -259,6 +460,7 @@ class SessionStateReactionToJobResultTests(TestCase):
         self.job_Y = make_job("Y")
         self.job_list = [self.job_A, self.job_R, self.job_X, self.job_Y]
         self.session = SessionState(self.job_list)
+        self.scratch_dir = TemporaryDirectory()
 
     def job_state(self, name):
         # A helper function to avoid overly long expressions
@@ -324,7 +526,8 @@ class SessionStateReactionToJobResultTests(TestCase):
         # session.
         result_R = JobResult({
             'job': self.job_R,
-            'io_log': ((0, 'stdout', "attr: value\n"),)
+            'io_log': make_io_log(((0, 'stdout', b"attr: value\n"),),
+                                  self.scratch_dir)
         })
         self.session.update_job_result(self.job_R, result_R)
         # The most obvious thing that can happen, is that the result is simply
@@ -372,12 +575,13 @@ class SessionStateReactionToJobResultTests(TestCase):
         # another proper record in that order.
         result_R = JobResult({
             'job': self.job_R,
-            'io_log': (
-                (0, 'stdout', "attr: value-1\n"),
-                (1, 'stdout', "\n"),
-                (1, 'stdout', "I-sound-like-a-broken-record\n"),
-                (1, 'stdout', "\n"),
-                (1, 'stdout', "attr: value-2\n"))
+            'io_log': make_io_log((
+                (0, 'stdout', b"attr: value-1\n"),
+                (1, 'stdout', b"\n"),
+                (1, 'stdout', b"I-sound-like-a-broken-record\n"),
+                (1, 'stdout', b"\n"),
+                (1, 'stdout', b"attr: value-2\n")),
+                self.scratch_dir)
         })
         # Since we cannot control the output of scripts and people indeed make
         # mistakes a warning is issued but no exception is raised to the
@@ -454,7 +658,8 @@ class SessionStateReactionToJobResultTests(TestCase):
         self.session.update_desired_job_list([self.job_A])
         result_R = JobResult({
             'job': self.job_R,
-            'io_log': ((0, 'stdout', 'attr: wrong value\n'),)
+            'io_log': make_io_log(((0, 'stdout', b'attr: wrong value\n'),),
+                                  self.scratch_dir)
         })
         self.session.update_job_result(self.job_R, result_R)
         # Now A is inhibited by FAILED_RESOURCE
@@ -471,7 +676,8 @@ class SessionStateReactionToJobResultTests(TestCase):
         # presented to a session that has some resources from that job already.
         result_R_old = JobResult({
             'job': self.job_R,
-            'io_log': ((0, 'stdout', "attr: old value\n"),)
+            'io_log': make_io_log(((0, 'stdout', b"attr: old value\n"),),
+                                  self.scratch_dir)
         })
         self.session.update_job_result(self.job_R, result_R_old)
         # So here the old result is stored into a new 'R' resource
@@ -480,7 +686,8 @@ class SessionStateReactionToJobResultTests(TestCase):
         # Now we present the second result for the same job
         result_R_new = JobResult({
             'job': self.job_R,
-            'io_log': ((0, 'stdout', "attr: new value\n"),)
+            'io_log': make_io_log(((0, 'stdout', b"attr: new value\n"),),
+                                  self.scratch_dir)
         })
         self.session.update_job_result(self.job_R, result_R_new)
         # What should happen here is that the R resource is entirely replaced
@@ -490,3 +697,149 @@ class SessionStateReactionToJobResultTests(TestCase):
         self.assertEqual(self.session._resource_map, expected_after)
 
     # TODO: add tests for local jobs
+
+    def tearDown(self):
+        self.scratch_dir.cleanup()
+
+
+class SessionStateLocalStorageTests(TestCase):
+
+    def setUp(self):
+        # session data are kept in XDG_CACHE_HOME/plainbox/.session
+        # To avoid resuming a real session, we have to select a temporary
+        # location instead
+        self._sandbox = tempfile.mkdtemp()
+        self._env = os.environ
+        os.environ['XDG_CACHE_HOME'] = self._sandbox
+
+    def job_state(self, name):
+        # A helper function to avoid overly long expressions
+        return self.session.job_state_map[name]
+
+    def test_persistent_save(self):
+        self.job_A = make_job("A")
+        self.job_list = [self.job_A]
+        self.session = SessionState(self.job_list)
+        result_A = JobResult({
+            'job': self.job_A,
+            'outcome': JobResult.OUTCOME_PASS,
+            'comments': 'All good',
+            'return_code': 0,
+            'io_log': ((0, 'stdout', "Success !\n"),)
+        })
+        session_json_text = """{
+            "_job_state_map": {
+                "A": {
+                    "_job": {
+                        "data": {
+                            "name": "A",
+                            "plugin": "dummy",
+                            "requires": null,
+                            "depends": null
+                        },
+                        "_class_id": "JOB_DEFINITION"
+                    },
+                    "_result": {
+                        "data": {
+                            "job": {
+                                "data": {
+                                    "name": "A",
+                                    "plugin": "dummy",
+                                    "requires": null,
+                                    "depends": null
+                                },
+                                "_class_id": "JOB_DEFINITION"
+                            },
+                            "outcome": "pass",
+                            "return_code": 0,
+                            "comments": "All good",
+                            "io_log": [
+                                [
+                                    0,
+                                    "stdout",
+                                    "Success !\\n"
+                                ]
+                            ]
+                        },
+                        "_class_id": "JOB_RESULT"
+                    },
+                    "_class_id": "JOB_STATE"
+                }
+            },
+            "_desired_job_list": [
+                {
+                    "data": {
+                        "name": "A",
+                        "plugin": "dummy",
+                        "requires": null,
+                        "depends": null
+                    },
+                    "_class_id": "JOB_DEFINITION"
+                }
+            ],
+            "_class_id": "SESSION_STATE"
+        }"""
+        self.session.open()
+        self.session.update_desired_job_list([self.job_A])
+        self.session.update_job_result(self.job_A, result_A)
+        self.session.persistent_save()
+        session_file = self.session.previous_session_file()
+        self.session.close()
+        self.assertIsNotNone(session_file)
+        with open(session_file) as f:
+            raw_json = json.load(f)
+            self.maxDiff = None
+            self.assertEqual(raw_json, json.loads(session_json_text))
+
+    def test_resume_session(self):
+        # All of the tests below are using one session. The session has four
+        # jobs, Job A depends on a resource provided by job R which has no
+        # dependencies at all. Both Job X and Y depend on job A.
+        #
+        # A -(resource dependency)-> R
+        #
+        # X -(direct dependency) -> A
+        #
+        # Y -(direct dependency) -> A
+        self.job_A = make_job("A", requires="R.attr == 'value'")
+        self.job_A_expr = self.job_A.get_resource_program().expression_list[0]
+        self.job_R = make_job("R", plugin="resource")
+        self.job_X = make_job("X", depends='A')
+        self.job_Y = make_job("Y", depends='A')
+        self.job_list = [self.job_A, self.job_R, self.job_X, self.job_Y]
+        # Create a new session (session_dir is empty)
+        self.session = SessionState(self.job_list)
+        result_R = JobResult({
+            'job': self.job_R,
+            'io_log': make_io_log(((0, 'stdout', b"attr: value\n"),),
+                                  self._sandbox)
+        })
+        result_A = JobResult({
+            'job': self.job_A,
+            'outcome': JobResult.OUTCOME_PASS
+        })
+        result_X = JobResult({
+            'job': self.job_X,
+            'outcome': JobResult.OUTCOME_PASS
+        })
+        # Job Y can't start as it requires job A
+        self.assertFalse(self.job_state('Y').can_start())
+        self.session.update_desired_job_list([self.job_X, self.job_Y])
+        self.session.open()
+        self.session.update_job_result(self.job_R, result_R)
+        self.session.update_job_result(self.job_A, result_A)
+        self.session.update_job_result(self.job_X, result_X)
+        self.session.persistent_save()
+        self.session.close()
+        # Create a new session (session_dir should contain session data)
+        self.session = SessionState(self.job_list)
+        self.session.open()
+        # Resume the previous session
+        self.session.resume()
+        # This time job Y can start
+        self.assertTrue(self.job_state('Y').can_start())
+        self.session.close()
+
+    def tearDown(self):
+        shutil.rmtree(self._sandbox)
+        os.environ = self._env
